# Make clean names csvs and population counts in 2010 for 2013 MSAs, 2018 MSAs, 1990 CZs

library(tidyverse)
library(stringr)

homedir = '<set to directory>'

setwd(homedir)
dir.create('output/msapop')

#Bring in block level population
nhgis = read.csv('data/nhgis0026_csv/nhgis0026_ds172_2010_block.csv')
nhgis$st = str_pad(nhgis$STATEA,width = 2,pad = '0')
nhgis$cnty = str_pad(nhgis$COUNTYA,width = 3,pad = '0')
nhgis$tract = str_pad(nhgis$TRACTA,width = 6,pad = '0')
nhgis$blkgrp = str_pad(nhgis$BLKGRPA,width = 1,pad = '0')
nhgis$blk = str_pad(nhgis$BLOCKA,width = 4,pad = '0')
nhgis$fips = paste(nhgis$st,nhgis$cnty,nhgis$tract,nhgis$blkgrp,nhgis$blk,sep = '')
nhgis$stcnty = paste(nhgis$st,nhgis$cnty,sep = '')

#Keep only stcounty, fips, and popultion 
nhgis = select(nhgis, fips, stcnty, pop = H7V001)

# Now loop through each of the MSA definitions, get their populations, get clean names


deffiles = c('msa_2013','msa_2018','cz_1990')

for(def in deffiles){
  mdef = read.csv(paste('data/reference/',def,'.csv',sep = ''))
  
  if(def %in% c('msa_2013','msa_2018')){
    mdef = mdef %>% select(msaraw = CBSA.Code,msaname = CBSA.Title,straw = FIPS.State.Code, cntyraw = FIPS.County.Code)
    mdef$st = str_pad(mdef$straw,2,pad = '0')
    mdef$cnty = str_pad(mdef$cntyraw,3,pad = '0')
    mdef$stcnty = paste(mdef$st, mdef$cnty, sep = '')
    
    #Take out PR
    mdef = filter(mdef, st != '72')
    }
  
  if(def %in% c('cz_1990')){
    mdef = mdef %>% select(msaraw = CZ90,msaname = Name.of.largest.place.in.Commuting.Zone,stcntyraw =County.FIPS.Code)
    mdef$stcnty = str_pad(mdef$stcntyraw,5,pad = '0')
    mdef$st = substr(mdef$stcnty,1,2)
    
    #Confirm no PR
    mdef = filter(mdef, st != '72')
    }
  

  mdef$msa = str_pad(mdef$msaraw,5,pad = '0')
  mdef = mdef %>% select(msa,msaname,stcnty)
  
  mpop = left_join(mdef, nhgis, by = 'stcnty')
  
  msapops = mpop %>% group_by(msa, msaname) %>% summarise(pop = sum(pop))
  
  write_csv(msapops, paste('output/msapop/pop_',def,'.csv',sep = ''))
}

