'''
Program to make shapefiles of each MSA
'''

import pandas as pd
import geopandas as gp
import os
from numpy import ceil


homedir = '<set to directory>'

os.chdir(homedir)


# Going to loop through all three MSA definitions

msadefs = ['msa_2013','msa_2018','cz_1990']

for mdef in msadefs:
    
    # Make directory for finished shapefiles
    try:
        os.stat("data/shapes_%s" %mdef)
    except:
        os.mkdir('data/shapes_%s' %mdef)

    # Bring in County to MSA lookup
    msalist = pd.io.parsers.read_csv('data/reference/%s.csv' %mdef, encoding ='ISO-8859-1') #, 'utf-8

    if mdef in ['msa_2013','msa_2018']:
        
        if mdef == 'msa_2018':
            startname = msalist.columns[0]
            msalist.rename(columns = {startname:'CBSA Code'},inplace = True)
        
        msalist['msa'] = msalist['CBSA Code'].apply(str)
        msalist['msaname'] = msalist['CBSA Title']
        msalist['fips_st'] = msalist['FIPS State Code'].apply(lambda x: "%02.f" %(x))
        msalist['fips_cnty'] = msalist['FIPS County Code'].apply(lambda x: "%03.f" %(x))
        msalist['stcnty'] = msalist['fips_st'] + msalist['fips_cnty']

    elif mdef in ['cz_1990']:
        msalist['stcnty'] = msalist['County FIPS Code'].apply(lambda x: "%05.f" %x)
        msalist['fips_st'] = msalist['stcnty'].apply(lambda x: x[:2])
        msalist['fips_cnty'] = msalist['stcnty'].apply(lambda x: x[2:5])
        msalist['msa'] = msalist['CZ90'].apply(str)
        msalist['msaname'] = msalist['Name of largest place in Commuting Zone'].apply(str)

    #Drop Puerto Rico
    msalist = msalist[msalist['fips_st'] != '72']


    # Order by state to avoid having to re-import stuff more than necessary
    #msadf = msalist.group_by('msa','fips_st').agg(len)
    # Sort by state then MSA ?
    msadf = msalist.sort_values(by = ['fips_st','msa','fips_cnty'])
    msadf = msadf.drop_duplicates('msa')
    msadf.reset_index(inplace =True)
    mlist = msadf['msa']
    stold = ''

    for msaint in mlist:
    
        msaname = msalist.loc[msalist['msa'] == msaint, 'msaname'].values[0]
        print(" %s: %s " % (msaint, msaname))
    
        #Look up the proper states for each MSA
        msacnts = msalist.loc[msalist['msa'] == msaint]
        msacntys = msacnts['stcnty']
    
        states = msacnts['fips_st'].unique()
        
        stnew = states[0]

        if stnew != stold:
    
        #Bring in centroids and area
        
            stblksall = gp.read_file("zip://data/raw_shapes/tl_2010_%s_tabblock10.zip" %states[0])
        #/tl_2010_%s_tabblock10 Folder/tl_2010_%s_tabblock10.shp" %(states[0],states[0]))
        cntsofint = msacnts[msacnts.fips_st == states[0]]['fips_cnty']

        stblks = stblksall[stblksall['COUNTYFP10'].isin(cntsofint.values)]
        
        if states.shape[0] > 1:
            for st in states[1:]:
                stblksadd =  gp.read_file("zip://data/raw_shapes/tl_2010_%s_tabblock10.zip" %st)
                cntsadd = msacnts[msacnts.fips_st == st]['fips_cnty']
                stblksadd = stblksadd[stblksadd['COUNTYFP10'].isin(cntsadd.values)]
                
                stblks = gp.GeoDataFrame(pd.concat([stblks,stblksadd]))
        
        stblks.crs = {'init': 'epsg:4269'}

        try:
            os.stat('data/shapes_%s/shp_%s_%s' %(mdef,mdef,msaint))
        except:
            os.mkdir('data/shapes_%s/shp_%s_%s' %(mdef,mdef,msaint))
            
        #Now reproject so buffers work
        '''
        #Figure out which UTM to use
        totbounds = stblks.geometry.total_bounds
        avglon = (totbounds[0] + totbounds[2]) / 2 
        zone = ceil((avglon + 180) / 6)
        utmepsg = utms[zone]
        assert len(utmepsg) == 5
    
        stblks['geometry'] = stblks.geometry.to_crs(epsg = utmepsg)
        stblks.crs = {'init': 'epsg:%s' %utmepsg}
'''

        stblks.to_file('data/shapes_%s/shp_%s_%s/shp_%s_%s_proj.shp' %(mdef,mdef,msaint,mdef,msaint))
        
        stold = stnew
        
#Update prj files
#for msaint in mlist:
#    shutil.copy('data/tabblock/tl_2010_01_tabblock10 Folder/tl_2010_01_tabblock10.prj','data/msashapes/msa_%s' %msaint)
#    os.rename('data/msashapes/msa_%s/tl_2010_01_tabblock10.prj' %msaint, 'data/msashapes/msa_%s/msa_%s.prj' %(msaint,msaint)

    
      #  os.mkdir('data/shapes_%s' %mdef)
       # if not os.path.exists('data/data/shapes_%s/shp_%s_%s' %(mdef,mdef,msaint)):
    
    
    