'''
Outputing csvs with job, area, lat/long, and population for all blocks by metro
'''

import pandas as pd
import numpy as np
import os
import sys

homedir = '<set to directory>'
os.chdir(homedir)

outdir = 'output'
#pathtolodes = '../jobs/data'
#pathtoshapes = '../reference/msashapes'

#New Modules
sys.path.append('code/modules')
from dataIO import dbf2df

#Set up 
try:
    os.stat("%s/blocks" %outdir)
except:
    os.mkdir("%s/blocks" %outdir)

try:
    os.stat("%s/zeroland" %outdir)
except:
    os.mkdir("%s/zeroland" %outdir)


##Bring in total population
natpop = pd.io.parsers.read_csv('data/nhgis0026_csv/nhgis0026_ds172_2010_block.csv', encoding = 'ISO-8859-1')

natpop['fips_st'] = natpop['STATEA'].apply(lambda x: "%02.f" %(x))
natpop['fips_cnty'] = natpop['COUNTYA'].apply(lambda x: "%03.f" %(x))
natpop['stcnty'] = natpop['fips_st'] + natpop['fips_cnty']
natpop['tract'] = natpop['TRACTA'].apply(lambda x: "%06.f" %(x))
#natpop['blkgrp'] = natpop['BLKGRPA'].apply(lambda x: "%01.f" %(x))
natpop['blk'] = natpop['BLOCKA'].apply(lambda x: "%04.f" %(x))
natpop['fips'] = natpop['fips_st'] + natpop['fips_cnty'] + natpop['tract'] + natpop['blk']
natpop['pop10'] = natpop['H7V001']
natpop = natpop[['fips_st','fips_cnty','stcnty','tract','blk','fips','pop10']]
# Loop through different conceptions

msadefs = ['cz_1990'] #'msa_2013', ,'msa_2018',''

for mdef in msadefs:
    
    #Make folder
    try: 
        os.stat('output/blocks/blocks_%s/' %mdef)
    except:
        os.mkdir('output/blocks/blocks_%s/' %mdef)

    
    #Bring in FIPS to abbreviation
    f2a = pd.io.parsers.read_csv('data/reference/fips2abbrev.csv')
    f2a['fips'] = f2a['fips'].apply(lambda x: "%02d" %x)
    
    #Bring in County to MSA list to look up st county
    msalist = pd.io.parsers.read_csv('data/reference/%s.csv' %mdef, encoding ='ISO-8859-1') #, 'utf-8
    
    if mdef in ['msa_2013','msa_2018']:
        
        if mdef == 'msa_2018':
            startname = msalist.columns[0]
            msalist.rename(columns = {startname:'CBSA Code'},inplace = True)
        
        msalist['msa'] = msalist['CBSA Code'].apply(str)
        msalist['msaname'] = msalist['CBSA Title']
        msalist['fips_st'] = msalist['FIPS State Code'].apply(lambda x: "%02.f" %(x))
        msalist['fips_cnty'] = msalist['FIPS County Code'].apply(lambda x: "%03.f" %(x))
        msalist['stcnty'] = msalist['fips_st'] + msalist['fips_cnty']

    elif mdef in ['cz_1990']:
        msalist['stcnty'] = msalist['County FIPS Code'].apply(lambda x: "%05.f" %x)
        msalist['fips_st'] = msalist['stcnty'].apply(lambda x: x[:2])
        msalist['fips_cnty'] = msalist['stcnty'].apply(lambda x: x[2:5])
        msalist['msa'] = msalist['CZ90'].apply(str)
        msalist['msaname'] = msalist['Name of largest place in Commuting Zone'].apply(str)


    #Drop Puerto Rico
    msalist = msalist[msalist['fips_st'] != '72']


    # Loop through all metros even though paper focuses on 100 largest
    #msadf = msalist.group_by('msa','fips_st').agg(len)
    # Sort by state then MSA ?
    msadf = msalist.sort_values(by = ['msa'])
    msadf = msadf.drop_duplicates('msa')
    msadf.reset_index(inplace =True)
    mlist = msadf['msa']
    #stold = ''
    
    #Pick up where we left off
    # msas = os.listdir('output/2018/%s/cbds' %place)
    # lastmsa = msas[len(msas)-1][5:10]
    # mlist = mlist[mlist > lastmsa]
    
    for msaint in mlist:

        msaname = msalist.loc[msalist['msa'] == msaint, 'msaname'].values[0]
        print(" %s: %s " % (msaint, msaname))
    
    
        #Look up the proper counties for it
        msacnts = msalist.loc[msalist['msa'] == msaint]
        msacntys = msacnts['stcnty']
    
        states = msacnts['fips_st'].unique()
        
        #Abbreviations
        stabrvs = f2a.loc[f2a['fips'].isin(states), 'abbrev'].str.lower()
    
        #Bring in data
        if stabrvs.values[0] == 'wy':
            styr = '2013'
        else: 
            styr = '2014'
            
        #WAC
        stwac = pd.io.parsers.read_csv('data/lehd/wac_01_%s/%s_wac_S000_JT01_%s.csv.gz' %(styr,stabrvs.values[0],styr))
                
        if stabrvs.shape[0] > 1:
            for st in stabrvs.iloc[1:]:
                
                if st == 'wy':
                    styr = '2013'
                else: 
                    styr = '2014'
                    
                wac = pd.io.parsers.read_csv( 'data/lehd/wac_01_%s/%s_wac_S000_JT01_%s.csv.gz' %(styr,st,styr))
                
                stwac = stwac.append(wac)
        
        stwac['fips'] = stwac['w_geocode'].apply(lambda x: "%015d" %x)
        
        #Keep total jobs and sectors
        jobcols = ['fips','C000']
        for secnum in range(1,21):
            jobcols.append('CNS%02d' %secnum)
            
        stwac = stwac[jobcols]
    
        #Take out fips to have list of job columns for later
        jobcols.remove('fips')
    
    
        # Get population
        pop = natpop[natpop.stcnty.isin(msacntys)]
        pop = pop[['fips','pop10']]
    
        #Bring in area and coordinates
        msaarea = dbf2df('data/shapes_%s/shp_%s_%s/shp_%s_%s.dbf' %(mdef,mdef,msaint,mdef,msaint)) #../reference/msashapes/msa_%s/msa_%s.dbf' %(msaint, msaint))
        msaarea['lat'] = pd.to_numeric(msaarea['INTPTLAT10'])
        msaarea['lon'] = pd.to_numeric(msaarea['INTPTLON10'])
        msaarea = msaarea[['ALAND10','GEOID10','lat','lon']]
        
        ## Make dataframe ##
        blks = msaarea.merge(stwac, how = 'left', left_on = 'GEOID10', right_on = 'fips')
        #blks = blks.merge(strac, how = 'left', left_on = 'GEOID10', right_on = 'fips')
        blks = blks.merge(pop, how = 'outer', left_on = 'GEOID10', right_on = 'fips')
        assert blks.shape[0] == pop.shape[0] #Same size hopefully
        assert blks.shape[0] == msaarea.shape[0] #Same size hopefully
        
        #Replace missings with zeros
        for job in jobcols:
            blks[job].fillna(0,inplace = True)
            
        
        #Make density in jobs per square km
        blks['jdens_raw'] = blks.C000 / (blks.ALAND10 / 1000000)
        blks['pdens'] = blks.pop10 / (blks.ALAND10 / 1000000)
        #blks['rdens'] = blks.racpop / (blks.ALAND10 / 1000000)
        blks.jdens_raw.fillna(0, inplace = True)
        blks.pdens.fillna(0, inplace = True)
        
        #If aland = 0 and jobs or pop > 0, export and set to 0 (to avoid making holes in shapefile)
        if(blks.jdens_raw.max() == np.inf):
            blks[blks.jdens_raw == np.inf].to_csv('output/zeroland/jobsnoland_%s_%s.csv' %(mdef,msaint))
            blks.C000[blks.jdens_raw == np.inf] = 0
            blks.jdens_raw[blks.jdens_raw == np.inf] = 0
    
        if(blks.pdens.max() == np.inf):
            blks[blks.pdens == np.inf].to_csv('output/zeroland/popnoland_%s_%s.csv' %(mdef,msaint))
            blks.pop10[blks.pdens == np.inf] = 0
            blks.pdens[blks.pdens == np.inf] = 0
       
        #Topcode job densities over 500k, and recompute jobs
        blks['jdens'] = blks.jdens_raw
        blks.jdens[blks.jdens_raw > 500000] = 500000
        blks['densfact'] = blks['jdens'] / blks['jdens_raw']
        blks.densfact.fillna(1,inplace = True)
        
        rawcols = []
        for job in jobcols:
            rawcols.append('%s_raw' %job)
            blks['%s_raw' %job] = blks[job]
            blks[job] = round(blks[job] * blks.densfact,2) #Limit to hundredths of jobs for shorter space
    
        #Prep for export
        exportcol = ['GEOID10','ALAND10','pop10','lat','lon','densfact','jdens','pdens']
        [exportcol.append(job) for job in jobcols]
        #[exportcol.append(raw) for raw in rawcols] #Don't think this is necessary because I have densfact
    
        blks = blks[exportcol]
        
        #Export
        blks.to_csv('output/blocks/blocks_%s/blks_%s_%s.csv' %(mdef,mdef,msaint), index = False)


