# Get count of blocks and total number of jobs in blocks that were topcoded because of their density
# Get statistics on average area of blocks by employment density and MSA

library(tidyverse)
library(stringr)
library(Hmisc)
library(scales)

homedir = '<set to directory>'
setwd(homedir)

msadefs = c('msa_2013','msa_2018','cz_1990')

for(mdef in msadefs){
  
  # Get list of files for all msas
  files = list.files(paste('output/blocks/blocks_',mdef,sep = ''))
  
  #Now go through each MSA, get its number, get count of blocks that were topcoded, get count of jobs that were deleted
  outp = NULL
  # allblks = NULL
  
  for(f in files){
    
    #Get MSA name
    mnum = str_extract(f,'[0-9]{4,5}\\.csv')
    # mnum = str_extract(f,'_[0-9]....\\.csv')
    mnum = str_extract(mnum,'[0-9]{4,5}')
    print(mnum)
    
    mblocks = read.csv(paste('output/blocks/blocks_',mdef,'/',f,sep = ''))
    
    #Drop unbuilt blocks
    mblocks = filter(mblocks, pop10 > 0 | C000 > 0)
    
    totblks = nrow(mblocks)
    
    topcodes = filter(mblocks, densfact < 1)
    
    #Get average size of block
    ablkmean = mean(mblocks$ALAND10)
    ablkmed = median(mblocks$ALAND10)
    ablkjmed = wtd.quantile(mblocks$ALAND10,mblocks$C000,probs = .5)
    
    #Median job density
    
    mblocks$jdens_clean = mblocks$C000 / (mblocks$ALAND10 / 1000000)
    mblocks$trujobs = mblocks$C000 / mblocks$densfact
    mblocks$jdens_raw = mblocks$C000 / mblocks$densfact / (mblocks$ALAND10 / 1000000)
    
    meddens_raw = wtd.quantile(mblocks$jdens_raw,mblocks$trujobs,probs = .5)
    meddens_clean = wtd.quantile(mblocks$jdens_raw,mblocks$C000,probs = .5)
    
    totjob_clean = sum(mblocks$C000)
    totjob_raw = sum(mblocks$trujobs)
    
    if(nrow(topcodes) == 0){
      blockstc = 0
      jobstc = 0
      tcemp = 0
      tcrawemp = 0
    }
    
    
    if(nrow(topcodes) > 0){
      blockstc = nrow(topcodes)
      topcodes$trujobs = topcodes$C000 / topcodes$densfact
      topcodes$empblock = topcodes$C000 > 2* topcodes$pop10
      topcodes$rawempblock = topcodes$trujobs > 2*topcodes$pop10
      tcemp = sum(topcodes$empblock)
      tcrawemp = sum(topcodes$rawempblock)
      jobstc = sum(topcodes$trujobs - topcodes$C000)
    }
    
    tcpct = jobstc / totjob_raw
    outrow = c(mnum,totjob_clean,totjob_raw,totblks,jobstc,blockstc,tcemp,tcrawemp,tcpct,ablkmed,ablkmean,ablkjmed,meddens_raw,meddens_clean)
    
    outp = rbind(outp,outrow)
    
    # allblks = rbind(allblks,mblocks)
  }
  outdf = data.frame(outp)
  names(outdf) = c('msa','totjobs_clean','totjobs_raw','totblks','tcjobs','tcblks','tc_empblocks','tc_emplblocksraw','tc_pct_of_raw','medarea','meanarea','medarea_jobwt','meddens_raw','meddens_clean')
  write_csv(outdf,paste('output/blocks/topcodes_',mdef,'.csv',sep = ''))
  
}


# Now look at land area
# abc = filter(allblks, ALAND10 > 0 | C000 == 0)
# medarea_nat = median(abc$ALAND10) #25192
# medarea_nat_job = wtd.quantile(abc$ALAND10,abc$C000,probs = .5) #90411
# 
# cor(abc$ALAND10,abc$jdens_clean,use = 'c')
# testl = lm(data = abc, formula = ALAND10 ~ jdens_clean)
# testl1 = lm(data = abc, formula = log(ALAND10+1) ~ log(jdens_clean+1))
# 
# pdf('output/2019/allblcks_cor.pdf') 
# 
# gplt = ggplot(abc, aes(x = jdens_clean, y = ALAND10)) +
#   geom_smooth(color = 'black') +
#   theme_bw() +
#   scale_y_continuous(name = 'Land area', labels = comma) +
#   scale_x_continuous(name = 'Density of employment', labels = comma) +
#   ggtitle('Area of Census blocks by density of employment, all 100 MSAs')
#    
# print(gplt)
# 
# dev.off()
# 
# pdf('output/2019/allblcks_cor_log.pdf') 
# 
# gplt = ggplot(abc, aes(x = jdens_clean, y = ALAND10)) +
#   geom_smooth(color = 'black') +
#   theme_bw() +
#   scale_y_continuous(name = 'Land area', labels = comma) +
#   scale_x_log10(name = 'Density of employment', labels = comma) +
#   ggtitle('Area of Census blocks by density of employment, all 100 MSAs')
# 
# print(gplt)
# 
# dev.off()
# 
# 
# pdf('output/2019/allblcks_cor_log2.pdf',height = 5) 
# 
# gplt = ggplot(abc, aes(x = jdens_clean, y = ALAND10)) +
#   geom_smooth(color = 'black') +
#   theme_bw() +
#   scale_y_log10(name = 'Land area', labels = comma) +
#   scale_x_log10(name = 'Density of employment', labels = comma) +
#   ggtitle('Area of Census blocks by density of employment, all 100 MSAs')
# 
# print(gplt)
# 
# dev.off()

