# Make dot density maps of Boston and Phoenix for Figure 1 in main text
# Unlike most analysis programs, this one is just using 2013 MSA definitions, because that is the focus of the main text

library(tidyverse)
library(rgdal) #Read OGR
library(ggthemes) #Theme_map
library(ggmap) # Google api
library(stringr)

homedir = '<set to directory>'
setwd(homedir)

register_google(key = '<sign up for google key>')

dir.create('output/dot_density')

# Import Boston shapefile
boston = readOGR(dsn = 'data/shapes_msa_2013/shp_msa_2013_14460/shp_msa_2013_14460.shp')

#Bring in data
blks = read.csv('output/blocks/blocks_msa_2013/blks_msa_2013_14460.csv') 
blks = blks %>% select(GEOID10,pop = pop10,jobs = C000) 

#Add block job and population data into the shapefile 
bmapd =  merge(boston@data, blks, by = 'GEOID10', sort=FALSE)
bos.dots = bmapd %>% select(pop, jobs) 

#Plot random points within each block - takes a while
bos.dfs <- lapply(names(bos.dots), function(x) {
  dotsInPolys(boston, as.integer(bos.dots[, x]), f="random")
})

#Turn into normal data frame
bdfs <- lapply(bos.dfs, function(x) {
  data.frame(coordinates(x)[,1:2])
})

#Select the jobs and population columns
bjobs = bdfs[[2]]
bpop = bdfs[[1]]

#Download basemap for Boston
bosbase = get_map(location = c(-71.060930,42.362482), maptype = 'toner-background', source = 'stamen', zoom = 11)

#Make partially transparent
bosbase_att = attributes(bosbase)
bosbase_trans <- matrix(adjustcolor(bosbase, 
                                    alpha.f = 0.15), 
                        nrow = nrow(bosbase))
attributes(bosbase_trans) = bosbase_att

#Map Boston jobs
jpeg('output/dot_density/map_boston_jobs.jpeg', width = 3000, height = 2312 )
p = ggmap(bosbase_trans) +
  geom_point(data = bjobs,aes(x = x, y=y ),pch = 20,size = .004, alpha = .08) +
  scale_y_continuous(limits = c(42.234,42.484),expand = c(0,0)) +
  theme_map()
print(p)
dev.off()

#Map Boston population 
jpeg('output/dot_density/map_boston_pop.jpeg', width = 3000, height = 2312 )
p = ggmap(bosbase_trans) +
  geom_point(data = bpop,aes(x = x, y=y ),pch = 20,size = .004, alpha = .08) +
  scale_y_continuous(limits = c(42.234,42.484),expand = c(0,0)) +
  theme_map()
print(p)
dev.off()


#Now do Phoenix
#shapefiles
phnx = readOGR(dsn = 'data/shapes_msa_2013/shp_msa_2013_38060/shp_msa_2013_38060.shp')

#Bring in block data
pblks = read.csv('output/blocks/blocks_msa_2013/blks_msa_2013_38060.csv') 
pblks = pblks %>% select(GEOID10,pop = pop10,jobs = C000) 
pblks$GEOID10 = str_pad(pblks$GEOID10,width = 15, pad = '0')

#Merge jobs and population into spatial dataframe
pmapd =  merge(phnx@data, pblks, by = 'GEOID10', sort = FALSE)
phx.dots = pmapd %>% select(pop, jobs) 

#Pick random points within each block
phx.dfs <- lapply(names(phx.dots), function(x) {
  dotsInPolys(phnx, as.integer(phx.dots[, x]), f="random")
})

#Turn into normal data frame
pdfs <- lapply(phx.dfs, function(x) {
  data.frame(coordinates(x)[,1:2])
})

#Get columns for plotting
pjobs = pdfs[[2]]
ppop = pdfs[[1]]

#Download background map
phxbase = get_map(location = c(-112.073840,33.448192), maptype = 'toner-background', source = 'stamen', zoom = 11)

#Set transparency
phxbase_att = attributes(phxbase)
phxbase_trans <- matrix(adjustcolor(phxbase,alpha.f = 0.15),nrow = nrow(phxbase))

attributes(phxbase_trans) = phxbase_att

# Output maps
jpeg('output/dot_density/map_phx_jobs.jpeg', width = 3000, height = 2312 )
p = ggmap(phxbase_trans) +
  geom_point(data = pjobs,aes(x = x, y=y ),pch = 20,size = .004, alpha = .08) +
  scale_y_continuous(limits = c(33.32191,33.60309),expand = c(0,0)) +
  theme_map()
print(p)
dev.off()

jpeg('output/dot_density/map_phx_pop.jpeg', width = 3000, height = 2312 )
p = ggmap(phxbase_trans) +
  geom_point(data = ppop,aes(x = x, y=y ),pch = 20,size = .004, alpha = .08) +
  scale_y_continuous(limits = c(33.32191,33.60309),expand = c(0,0)) +
  theme_map()
print(p)
dev.off()


