#Get summary statistics of total employment, employemnt in BDs, largest BD size, number of large BDS

library(Hmisc)
library(ggplot2)
library(scales)
library(stringr)
library(dplyr)
library(reshape2)

homedir = '<set to directory>'
setwd(homedir)

# Loop through MSA definitions
msadefs = c('msa_2013','msa_2018','cz_1990')

for(mdef in msadefs){
  mdef_clean = str_split(mdef,'_',simplify = T)[,1] %>% str_to_upper()
  
  #Get MSA stats
  msas = read.csv(paste('output/msapop/pop_',mdef,'.csv',sep = ''))
  msas$shortname = str_split(msas$msaname,'[-,/]',simplify = T)[,1]
  msas$st = str_split(msas$msaname,'[,]',simplify = T)[,2]
  msas$st = str_replace(msas$st, ' ','')
  msas$st = str_split(msas$st,'-',simplify = T)[,1]
  msas$shortname_st = paste(msas$shortname,msas$st, sep = ', ')
  
  #Identify the 100 largest MSAs
  m100 = msas %>% arrange(-pop)
  m100 = m100[1:100,'msa']
  
  #If there are multiple specifications, can loop through them here
  for(place in c('bizdist_b15_r2.0')){ #,'bizdist_b15_d0.5','bizdist_b15_r2.0_scan3'
    
  sumstats = NULL

  for(cb in msas$msa){
    if(cb == '34114') next()
    
    msaname = as.character(msas[msas$msa == cb,'msaname'])
    sname = msas[msas$msa == cb,'shortname']
    print(paste(cb,msaname,sname))
    
    #Bring in categorized business districts (which include one row for all mixed and residential blocks combined)
    cbds = read.csv(paste('output/',place,'/',mdef,'/bd_cats/bd_cats_',mdef,'_',cb,'.csv',sep = ''))
    
    #Get total employment
    totemp = sum(cbds$jobs)
    emps = filter(cbds,cat == 'emp')
    empemp = sum(emps$jobs)
    pctemp = empemp / totemp
    
    #Number of BDs with more than 1% of total MSA employment
    emps$pct = emps$jobs / totemp
    ngt1pct = filter(emps, pct > .01) %>%nrow()
    
    #Number over 20000 jobs
    ngt20k = filter(emps, jobs > 20000) %>% nrow()
    
    #Biggest CBD percent
    biggest = emps[1,'pct']
  
    #Add to summary statistics dataframe
    statrow = c(cb,msaname,totemp,pctemp,biggest,ngt1pct,ngt20k)
    sumstats = rbind(sumstats,statrow)
    
  }

  #Output all MSAs
  sumdf = data.frame(sumstats)
  names(sumdf) = c('msa','msaname','totemp','pctemp','biggest','ngt1','ngt20k')
  write.csv(sumdf,paste('output/',place,'/',mdef,'/table_s1_msa_sumstats.csv',sep = ''),row.names = F)

  #Output top 100
  sumdf100 = filter(sumdf, msa %in% m100) 
  write.csv(sumdf100,paste('output/',place,'/',mdef,'/table_s1_msa_sumstats_top100.csv',sep = ''),row.names = F)
  
  
    }
  }