#Make dot maps of different CBDs

library(maptools)
library(rgeos)
library(tidyverse)
library(rgdal)
library(ggthemes)
library(ggmap)
library(stringr)
library(Hmisc)
library(scales)

homedir = '<set to directory>'
setwd(homedir)

#Set up Google
apikey = '<sign up for google key>'
register_google(key = apikey)

# Loop through MSA definitions
msadefs = c('msa_2013','msa_2018','cz_1990')

for(mdef in msadefs){
  mdef_clean = str_split(mdef,'_',simplify = T)[,1] %>% str_to_upper()
  
  #Get MSA stats
  msas = read.csv(paste('output/msapop/pop_',mdef,'.csv',sep = ''))
  msas$shortname = str_split(msas$msaname,'[-,/]',simplify = T)[,1]
  msas$st = str_split(msas$msaname,'[,]',simplify = T)[,2]
  msas$st = str_replace(msas$st, ' ','')
  msas$st = str_split(msas$st,'-',simplify = T)[,1]
  msas$shortname_st = paste(msas$shortname,msas$st, sep = ', ')
  
  #Loop through MSAs in order of population just in case we run out of API calls
  msas = msas %>% arrange(-pop)
  
  #Loop through MSAs
  for(i in 1:nrow(msas)){
    
    msa = msas[i,'msa']
    sname = msas[i,'shortname_st']
    longname = msas[i,'msaname']
    print(paste(msa, sname,i))
    
  
    #Get basemap first to limit calls to google
    basemap= get_map(location = sname, maptype = 'toner-lite', source = 'stamen', zoom = 10) #-73.984010,40.755038
    # basemapz= get_map(location = sname, maptype = 'satellite', source = 'google', zoom = 11) #-73.984010,40.755038
    
    # Can loop through alternate specifications here
    for(place in c('b15_r2.0')){ #,'b15_d0.5','b15_r2.0_scan3'
      
      dir.create(paste('output/bizdist_',place,'/',mdef,'/bd_maps/maps_',mdef,'_',msa,'_',sname,sep = ''))
        
      cbds = read.csv(paste('output/bizdist_',place,'/',mdef,'/bd_cats/bd_cats_',mdef,'_',msa,'.csv',sep = ''))
      cbds = filter(cbds, cat == 'emp')
      
      pdf(paste('output/bizdist_',place,'/',mdef,'/bd_maps/maps_',mdef,'_',msa,'_',sname,'/bd_dots_',msa,'_',sname,'_dots.pdf',sep = ''), height = 7, width = 6)
      p = ggmap(basemap) + #, darken = c(.3, 'white'),color = 'bw') +
        geom_point(data = cbds, aes(x = lon, y = lat, size = jobs), alpha = .3) +
        scale_size_area(max_size = 35,limits = c(1,1500000), breaks = c(10000,100000,500000), labels = comma, name = 'Business district employment') +
        labs(title = paste('Business districts of the',longname,mdef_clean), caption = 'Map tiles by Stamen Design, under CC BY 3.0. Data by OpenStreetMap, under CC BY SA') +
        theme(axis.text = element_blank(),axis.ticks = element_blank(),
              axis.title = element_blank(),plot.caption = element_text(hjust = 0,size = 7),
              legend.position = 'bottom',legend.key = element_rect(fill = 'white'))
      print(p)
      dev.off()
      
      # Make clean versions of Atlanta and Denver for supplementary materials
      if(msa %in% c('12060','19740')){
        pdf(paste('output/bizdist_',place,'/',mdef,'/bd_maps/figS67_bd_dots_',msa,'_',sname,'_dots.pdf',sep = ''), height = 7, width = 6)
        p = p + ggtitle('')
        print(p)
        dev.off()
      }
      
    }
  }
}
    