#Use Google maps API to download central points for cities
# This doesn't work without a Google API key 

library(maptools)
library(rgeos)
library(tidyverse)
library(rgdal)
library(ggthemes)
library(ggmap)
library(stringr)
library(Hmisc)
library(scales)

homedir = '<set to directory>'
setwd(homedir)

register_google(key = '<sign up for google key>')

# Loop through MSA definitions
msadefs = c('msa_2013','msa_2018','cz_1990')

for(mdef in msadefs){
  mdef_clean = str_split(mdef,'_',simplify = T)[,1] %>% str_to_upper()
  
  #Get MSA stats
  msas = read.csv(paste('output/msapop/pop_',mdef,'.csv',sep = ''))
  msas$shortname = str_split(msas$msaname,'[-,/]',simplify = T)[,1]
  msas$st = str_split(msas$msaname,'[,]',simplify = T)[,2]
  msas$st = str_replace(msas$st, ' ','')
  msas$st = str_split(msas$st,'-',simplify = T)[,1]
  msas$shortname_st = paste(msas$shortname,msas$st, sep = ', ')
  
  #Geocode
  googlell = NULL
  for(i in 1:nrow(msas)){
    msa = msas[i,'msa']
    sname = msas[i,'shortname_st']
    
    gcode = geocode(sname,source = 'google',output = 'latlon')
    googlell = rbind(googlell,c(msa,sname,gcode$lon,gcode$lat))
  
  }
  
  goout = data.frame(googlell)
  names(goout) = c('msa','name','glon','glat')
  goout$glat = as.numeric(as.character(goout$glat))
  goout$glon = as.numeric(as.character(goout$glon))
  write.csv(goout,paste('output/msapop/metcents_google_',mdef,'.csv',sep = ''),row.names = F)
  
  # The connection sometimes breaks off, so the program will keep looping until you fill everything in
  while(sum(is.na(goout$glat)) > 0){
    
  newmsas = goout %>% filter(is.na(glon)) %>% select(msa) %>% unlist()
  
  for(j in newmsas){
    sname = goout[goout$msa == j,'name'] %>% as.character()
    
    gcode = geocode(sname,source = 'google',output = 'latlon')
    goout[goout$msa == j,'glon'] = gcode$lon
    goout[goout$msa == j,'glat'] = gcode$lat
  }
  
  print(sum(is.na(goout$glon)))
  }
  write.csv(goout,paste('output/msapop/metcents_google_',mdef,'.csv',sep = ''),row.names = F)
}