#Compare CBD as identified by Google with that identified by the algorithm

library(ggmap)
library(tidyverse)
library(fields)
library(ggthemes)

homedir = '<set to directory>'
setwd(homedir)

# Loop through MSA definitions
msadefs = c('msa_2013','msa_2018','cz_1990')

for(mdef in msadefs){
  mdef_clean = str_split(mdef,'_',simplify = T)[,1] %>% str_to_upper()
  
  #Get MSA stats
  msas = read.csv(paste('output/msapop/pop_',mdef,'.csv',sep = ''))
  msas$shortname = str_split(msas$msaname,'[-,/]',simplify = T)[,1]
  msas$st = str_split(msas$msaname,'[,]',simplify = T)[,2]
  msas$st = str_replace(msas$st, ' ','')
  msas$st = str_split(msas$st,'-',simplify = T)[,1]
  msas$shortname_st = paste(msas$shortname,msas$st, sep = ', ')
  
  #Identify the 100 largest MSAs
  m100 = msas %>% arrange(-pop)
  m100 = m100[1:100,'msa']
  
  #If there are multiple specifications, can loop through them here
  for(place in c('bizdist_b15_r2.0')){ #bizdist_b15_r2.0_scan3
    
    #Make map directory
    dir.create(paste('output/',place,'/',mdef,'/metro_centers/mc_maps',sep = ''))

    algo = read.csv(paste('output/',place,'/',mdef,'/metro_centers/metcents_',mdef,'_',place,'.csv',sep = ''))
    google = read.csv(paste('output/msapop/metcents_google_',mdef,'.csv',sep = ''))

    comp = merge(algo %>% select(msa,lat,lon),google, by  = 'msa',all = T)
    
    # Confirm that same MSAs are in both
    if(mdef != 'cz_1990') stopifnot(nrow(comp) == nrow(algo))
    if(mdef == 'cz_1990') stopifnot(nrow(comp) == nrow(algo) + 1)

    #Use rdist to get great circle distance between different estimated cbds
    comp$dist = NA

    for(m in comp$msa){
      mdta = filter(comp, msa == m)
      dist = rdist.earth(x1 = mdta %>% select(lon,lat),x2 = mdta %>% select(glon,glat),miles = T)[1]
      comp[comp$msa == m,c('dist')] = c(dist)
    }
    
outdf = comp %>% select(msa,name,lat,lon,glat,glon,dist)
write.csv(outdf, paste('output/',place,'/',mdef,'/metro_centers/table_s4_compare_cbdloc.csv',sep = ''),row.names = F)

out100 = outdf %>% filter(msa %in% m100)
write.csv(out100, paste('output/',place,'/',mdef,'/metro_centers/table_s4_compare_cbdloc_top100.csv',sep = ''),row.names = F)

outallclean = comp %>% select(msa,name,lat,lon)
write.csv(outallclean, paste('output/',place,'/',mdef,'/metro_centers/cbds_clean_all.csv',sep = ''),row.names = F)

out100clean = outallclean %>% filter(msa %in% m100)
write.csv(out100clean, paste('output/',place,'/',mdef,'/metro_centers/cbds_clean_top100.csv',sep = ''),row.names = F)

#Now make maps
for(m in comp$msa[1:nrow(comp)]){ #Note that you have to run this a bunch of times because the query limit crashes it
  mdta = filter(comp, msa == m)
  sname = as.character(mdta$name)
  basemap= get_map(location = sname, maptype = 'toner-lite', source = 'stamen', zoom = 10) #-73.984010,40.755038

  #Make df
  mcdf = rbind(mdta %>% select(lon,lat) %>% unlist(),mdta %>% select(glon,glat) %>% unlist()) %>% data.frame()
  mcdf$Method = c('Algorithm','Google')

  pdf(paste('output/',place,'/',mdef,'/metro_centers/mc_maps/mc_comp_',m,'_',sname,'.pdf',sep = ''), width = 4, height = 4 )
  p = ggmap(basemap, darken = c(.2, 'white'),color = 'bw') +
    geom_point(data = mcdf, aes(x = lon, y = lat, color = Method, pch = Method)) +
    scale_color_brewer(type = 'qual',palette = 'Set1') +
    theme_map()
  print(p)
  dev.off()

}
  }
}

#Summary statistics
sum(comp$dist < 1) #69 / 100 or 461 / 917
sum(comp$dist < 2) #81 / 100 or 647 / 917
sum(comp$dist < 5) #88 / 100 or 785 / 917
sum(comp$dist < 10) #95 / 100 or 825 / 917

sum(out100$dist < 1) #68 / 100 or 461 / 917
sum(out100$dist < 2) #80 / 100 or 647 / 917
sum(out100$dist < 5) #88 / 100 or 785 / 917
sum(out100$dist < 10) #95 / 100 or 825 / 917

#In some cases mine are arguably more true than Googles. Of those >2 miles apart
#SF - theirs is by Civic Center, mine is financial district (but only 1 mile)
#Bridgeport - Mine is Stamford, which is the bigger business district
#Cape Coral - Mine is Fort Myers
#Deltona - theirs is deltona proper, mine is daytona beach, which is bigger
#North Port FL - mine is branderton, theirs is northport random spot
#NY - theirs is downtown, mine is midtown
#Palm Bay FL - theirs is Palm Bay, mine is Melbourne
#Scranton - theirs is Scranton, mine is Wilkes Barre
#Virginia Beach - theirs is VA beach, mine is Norfolk


#Ones where mine is definitely worse:
#Bakersfield
#Chattanooga
#Dallas
#El Paso - I'm in an army base
#Oxnard
#Phoenix
#Raleigh
#Rochester
#Salt Lake City
#Toledo
#Witchita
