#Confirm statistics mentioned in the paper
library(dplyr)
library(reshape2)
library(ggplot2)
library(stringr)
library(scales)

homedir = '<set to directory>'
setwd(homedir)


#Page 8 of word doc - total number of primary jobs in LODES data
rawdir = 'data/lehd/wac_01_2014/'
dtasets = list.files(rawdir)

#Loop through all states and bring in raw files
allstates = NULL

for(st in dtasets){
  dta = read.csv(paste(rawdir,st,sep = ''))
  sttotal = sum(dta$C000)
  allstates = rbind(allstates,c(st,sttotal))
}

#Add Wyoming 2013
dta = read.csv('data/lehd/wac_01_2013/wy_wac_S000_JT01_2013.csv.gz')
sttotal = sum(dta$C000)
allstates = rbind(allstates, c('wy',sttotal))

#Convert format
natjobs = data.frame(allstates)
names(natjobs) = c('state','primaryjobs')
natjobs$primaryjobs = as.numeric(as.character(natjobs$primaryjobs))

#Sum
sum(natjobs$primaryjobs) #125697883
sum(natjobs$primaryjobs) / (146308 *1000) # 146308k is Civilian Employment Level in 2014 from FRED CE16OV - averaged by year


## Number of blocks in Los Angeles MSA - cite on page 8
lablks = read.csv('output/blocks/blocks_msa_2013/blks_msa_2013_31080.csv')
nrow(lablks) #146462 - this is after dropping zero land but not after dropping unbuilt
sum(lablks$ALAND10 == 0 & lablks$C000 >0)
sum(lablks$pop10 == 0 & lablks$C000 == 0)

## Number of blocks in top 100 MSAs that are topcoded - footnote 2
msapops = read.csv('output/msapop/pop_msa_2013.csv')
m100 = msapops %>% arrange(-pop)
m100codes = m100[1:100,'msa']

topcodes = read.csv('output/blocks/topcodes_msa_2013.csv')
top100tc = topcodes %>% filter(msa %in% m100codes)
sum(top100tc$tcblks) #689
sum(top100tc$tcjobs) #1587804
sum(top100tc$totblks) #689
sum(top100tc$totjobs_raw) #87880668
1587804/87880668 #0.01806773


## Percentage of all primary jobs in top 100 MSAs - page 9
sum(top100tc$totjobs_raw) #87880668
sum(top100tc$totjobs_clean) #86292864
sum(natjobs$primaryjobs) #125697883
87880668/125697883 #0.699142
86292864/125697883 #0.6865101 - use this because it's my sample

## Percentage of population in top 100 MSAs - page 9
mpop = filter(msapops, msa %in% m100codes)
sum(mpop$pop) #203854846
203854846 / 308745538 #Population in 2010 Census. From summing states, the 308 doesn't include PR https://en.wikipedia.org/wiki/2010_United_States_Census
#0.6602682


## Median residential and jobs density - on page 10 claim that median job is denser than median person in all but 5 metros
mstats = read.csv('output/block_analysis/msa_2013/blockstats_msa_2013.csv')
mstats = filter(mstats, msa %in% m100codes)
sum(mstats$med_jdens > mstats$med_pdens) #95
sum(mstats$med_jdens < mstats$med_pdens) #5

# Correlation - SM page 6
cor(mstats$med_jdens,mstats$med_pdens) #0.7192453


## Percent of jobs and residents on densest 10% land in Boston, page 11 - see get_metro_centers.R

## Median MSA percent with zero jobs
median(mstats$nojobs_area) #0.5140184
median(mstats$nojobs_blks) #0.6483406
max(mstats$nopop_area) #0.1863739

## Percent of jobs, land area, 
summary(mstats$jobs_emp) # Median is 0.7646
summary(mstats$area_emp) # Median is 0.05980
summary(mstats$jobs_mix) # Median is 0.1290
summary(mstats$area_mix) # Median is 0.05383
summary(mstats$jobs_res) # Median is 0.10097
summary(mstats$area_res) # Median is 0.8850


## Clustered-ness, discussed on page 14
mclust = read.csv('output/bizdist_b15_r2.0/msa_2013/bd_cluster_sim.csv')
min(mclust$jt5) #0.4183806 - percent of all jobs in clusters of >5
max(mclust$jt5) #0.7197351
min(mclust$pt5) #0.5907099 - percent of employment jobs in clusters of >5
max(mclust$pt5) #0.8902408 - percent of employment jobs in clusters of >5

sum(mclust$t1) #59637031
sum(mclust$t5) #51366220
sum(mclust$etot) #65955008
59637031/65955008 #0.9042078
51366220/65955008 #0.778807

#Clustered-ness in supplementary material page 
median(mclust$pc5) #0.06343281 Median across MSAs of p95 draw 
median(mclust$pt5) #0.7666823 Median across MSAs of observed value
max(mclust$pc5) #0.195211 Max across MSAs of p95 draw 


## New York densities cited on page 15
nycbd = read.csv('output/bizdist_b15_r2.0/msa_2013/bd_cats/bd_cats_msa_2013_35620.csv')

# Midtown mean density
filter(nycbd, cbd == 13) %>% select(tdens) #159130.1
filter(nycbd, cbd == 13) %>% select(jobs) #1112861.0

# Harmon cove is district 64
filter(nycbd, cbd == 64) %>% select(tdens) #3302.301
filter(nycbd, cbd == 64) %>% select(jobs) #31763
which(nycbd$cbd == 64) #15 minus mixed and res = 13th largest


#What percent of all jobs are employment districts - SM page 12
sum(filter(nycbd, cat == 'emp')$jobs) #5476206
sum(nycbd$jobs) #7757624
5476206/7757624 #0.7059128

#What about using G&S approach?
nycgs = read.csv('output/bizdist_b15_d0.5/msa_2013/bd_cats/bd_cats_msa_2013_35620.csv')
sum(filter(nycgs, cat == 'emp')$jobs) #4807278
sum(nycgs$jobs) #7757624
4807278/7757624 #0.6196843

## How many large bds - page 15-16
bdstats = read.csv('output/bizdist_b15_r2.0/msa_2013/table_s1_msa_sumstats_top100.csv')
summary(bdstats$ngt1)
quantile(bdstats$ngt1,probs = c(.05,.1,.9,.95))
sum(bdstats$ngt1 >=10 & bdstats$ngt1 <= 15) #68
median(bdstats$biggest)
summary(bdstats$biggest) #0.03134 - 0.30461
bdstats %>% arrange(-biggest) %>% head() #San Jose
bdstats %>% arrange(biggest) %>% head() #Detroit


# Categories 
bdcats = read.csv('output/bizdist_b15_r2.0/msa_2013/bdcats_national_msa_2013.csv')

# Minimum fraction of employment in suburban strips - page 12 of pdf
bdcats %>% filter(msa %in% m100codes, catdet == 'suburb') %>% arrange(jobs_pct) %>% head() #0.2364084
filter(msapops, msa == 46520) #Honolulu

# Number and total importance of urban cores across msas - page 18-19
bdcats = filter(bdcats, msa %in% m100codes, catdet == 'core')

# Number of cities with no urban core - page 12 of pdf
nrow(bdcats) #97
summary(bdcats$jobs_pct)
bdcats %>% arrange(-jobs_pct) %>% head() #Honolulu with 37%, then Washington DC with 29%

# NYC urban cores
filter(bdcats, msa == '35620', catdet == 'core') %>% select(num,jobs_pct) #104


# 3M HQ in St Paul - page 18
mspbd = read.csv('output/bizdist_b15_r2.0/msa_2013/bd_cats/bd_cats_msa_2013_33460.csv')
filter(mspbd, cbd == 9)
which(mspbd$cbd == 9) #13 - 2 = 11

# How many MSAs have no urban core >1% emp?
metcents = read.csv('output/bizdist_b15_r2.0/msa_2013/metro_centers/metcents_msa_2013_bizdist_b15_r2.0.csv')
filter(metcents, msa %in% m100codes, catdet != 'core') %>% nrow() #7

